{ Office 97 Controls version 1.5, copyright  A. Meeder, december 1996 - may 1997

  ** It's freeware, so you don't have to pay me, but I do not guarantee the working of it

  The idea came from the personal assistent of MS Office 97. To use these components:

  Special properties of TOfficeButton:
  * BorderColor, color of the border when the button has the mouse
  * BorderWidth, basic width of the border (1 .. 3)
  * ButtonColor, color of button-face when Transparent is false
  * RoundSize, used to change the corners of the button
  * Transparent, default true, or false

  Special events of TOfficeButton:
  * OnMouseEnter
  * OnMouseLeave

  Sepcial properties of TOfficeLabel:
  * LabelColor, color of label-face when Transparent is false
  * LEDType, (RoundLED, TriLEDUp, TriLEDDown)
  * Transparent, default true, or false

  Special events of TOfficeLabel:
  * OnMouseEnter
  * OnMouseLeave

  If you have suggestions about TOButton or make changes, please mail me: ameeder@dds.nl

  Version:
      1.5: Added two events 'OnMouseEnter/OnMouseLeave' (Delphi 1.0 version available)
      1.4: Added 'Enabled' property, changed *.RES-name, fixed some code...
      1.3: Reduced the flickering when you have several Officelabels, -buttons and move over them
      1.2: Added 'Transparent' property
      1.1: Fixed some bugs
      1.0: First release 
}

unit OContrls;

interface

uses
  {$IFDEF WIN32}
    Windows,
  {$ELSE}
    WinProcs, WinTypes,
  {$ENDIF}
    SysUtils, Messages, Classes, Graphics, Controls;

type
  TBorderWidth = 1..3;

  TOfficeButton = class(TGraphicControl)
  private
    FBorderColor: TColor;
    FBorderWidth: TBorderWidth;
    FButtonColor: TColor;
    FCaption: TCaption;
    FEnabled: boolean;
    FFont: TFont;
    FGlyph: TBitmap;
    FRoundSize: integer;
    FTransparent: boolean;

    FMouseEnter: TNotifyEvent;
    FMouseLeave: TNotifyEvent;

    HasMouse: boolean;
    IsMouseDown: boolean;
    ExtraSpace: word;

    procedure SetBorderColor(value: TColor);
    procedure SetBorderWidth(value: TBorderWidth);
    procedure SetButtonColor(value: TColor);
    procedure SetCaption(value: TCaption);
    procedure SetEnabled(value: boolean);
    procedure SetFont(value: TFont);
    procedure SetRoundSize(value: integer);
    procedure SetTransparent(value: boolean);
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure Click; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure WMLButtonDblClick(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMMButtonDblClick(var Message: TWMMButtonDown); message WM_MBUTTONDOWN;
    procedure WMRButtonDblClick(var Message: TWMRButtonDown); message WM_RBUTTONDOWN;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property BorderColor: TColor read FBorderColor write FBorderColor default clGray;
    property BorderWidth: TBorderWidth read FBorderWidth write SetBorderWidth default 1;
    property ButtonColor: TColor read FButtonColor write SetButtonColor default clSilver;
    property Caption: TCaption read FCaption write SetCaption;
    property Enabled: boolean read FEnabled write SetEnabled default true;
    property Font: TFont read FFont write SetFont;
    property RoundSize: integer read FRoundSize write SetRoundSize default 15;
    property Transparent: boolean read FTransparent write SetTransparent default true;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseEnter: TNotifyEvent read FMouseEnter write FMouseEnter;
    property OnMouseLeave: TNotifyEvent read FMouseLeave write FMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;


  TLEDType = (RoundLED, TriLEDDown, TriLEDUp);

  TOfficeLabel = class(TGraphicControl)
  private
    FCaption: TCaption;
    FEnabled: boolean;
    FFont: TFont;
    FGlyph: TBitmap;
    FLabelColor: TColor;
    FLedType: TLEDType;
    FTransparent: boolean;

    FMouseEnter: TNotifyEvent;
    FMouseLeave: TNotifyEvent;

    HasMouse, IsMouseDown, IsClicked: boolean;

    procedure SetCaption(value: TCaption);
    procedure SetEnabled(value: boolean);
    procedure SetFont(value: TFont);
    procedure SetLabelColor(value: TColor);
    procedure SetLEDType(value: TLEDType);
    procedure SetTransparent(value: boolean);
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure Click; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure WMLButtonDblClick(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMMButtonDblClick(var Message: TWMMButtonDown); message WM_MBUTTONDOWN;
    procedure WMRButtonDblClick(var Message: TWMRButtonDown); message WM_RBUTTONDOWN;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Caption: TCaption read FCaption write SetCaption;
    property Enabled: boolean read FEnabled write SetEnabled default true;
    property Font: TFont read FFont write SetFont;
    property LabelColor: TColor read FLabelColor write SetLabelColor default clSilver;
    property LEDType: TLEDType read FLEDType write SetLEDType default RoundLED;
    property Transparent: boolean read FTransparent write SetTransparent default True;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseEnter: TNotifyEvent read FMouseEnter write FMouseEnter;
    property OnMouseLeave: TNotifyEvent read FMouseLeave write FMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

{$R OCONTRLS.RES}

procedure TOfficeButton.SetBorderColor(value: TColor);
begin
  if Value <> FBorderColor then
  begin
    FBorderColor := Value;
    Repaint;
  end;
end;

procedure TOfficeButton.SetBorderWidth(value: TBorderWidth);
begin
  if Value <> FBorderWidth then
  begin
    FBorderWidth := Value;
    Repaint;
  end;
end;

procedure TOfficeButton.SetButtonColor(value: TColor);
begin
  if Value <> FButtonColor then
  begin
    FButtonColor := Value;
    Repaint;
  end;
end;

procedure TOfficeButton.SetCaption(value: TCaption);
begin
  if Value <> FCaption then
  begin
    FCaption := Value;
    Repaint;
  end;
end;

procedure TOfficeButton.SetEnabled(value: boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    HasMouse := false;
    IsMouseDown := false;
    ExtraSpace := 0;
    Repaint;
  end;
end;

procedure TOfficeButton.SetFont(value: TFont);
begin
  if Value <> FFont then
  begin
    FFont.Assign(Value);
    Repaint;
  end;
end;

procedure TOfficeButton.SetRoundSize(value: integer);
begin
  if Value <> FRoundSize then
  begin
    FRoundSize := Value;
    Repaint;
  end;
end;

procedure TOfficeButton.SetTransparent(value: boolean);
begin
  if value <> FTransparent then
  begin
    FTransparent := value;
    Repaint;
  end;
end;

procedure TOfficeButton.CMMouseEnter(var Message: TMessage);
begin
  if Enabled then
  begin
    HasMouse := true;
    if IsMouseDown then ExtraSpace := 1 else ExtraSpace := 0;
    Repaint;
    if Assigned(FMouseEnter) then FMouseEnter(Self);
  end;
end;

procedure TOfficeButton.CMMouseLeave(var Message: TMessage);
begin
  if Enabled then
  begin
    HasMouse := false;
    ExtraSpace := 0;
    Repaint;
    if Assigned(FMouseLeave) then FMouseLeave(Self);
  end;
end;

procedure TOfficeButton.WMMouseMove(var Message: TWMMouseMove);
var
  CurrentPoint: TPoint;
begin
  if Enabled then
  begin
    CurrentPoint.X := Message.XPos;
    CurrentPoint.Y := Message.YPos;
    if not HasMouse and PtInRect(GetClientRect, CurrentPoint) then
    begin
      HasMouse := true;
      Repaint;
    end;
  end;
  inherited;
end;

procedure TOfficeButton.WMLButtonDblClick(var Message: TWMLButtonDown);
begin
  IsMouseDown := false;
  if Enabled then inherited;
end;

procedure TOfficeButton.WMMButtonDblClick(var Message: TWMMButtonDown);
begin
  IsMouseDown := false;
  if Enabled then inherited;
end;

procedure TOfficeButton.WMRButtonDblClick(var Message: TWMRButtonDown);
begin
  IsMouseDown := false;
  if Enabled then inherited;
end;

procedure TOfficeButton.Click;
begin
  if Enabled then inherited Click;
end;

procedure TOfficeButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FEnabled then
  begin
    HasMouse := true;
    IsMouseDown := true;
    ExtraSpace := 1;
    Repaint;
    inherited MouseDown(Button, Shift, X, Y);
  end;
end;

procedure TOfficeButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FEnabled then
  begin
    IsMouseDown := false;
    ExtraSpace := 0;
    Repaint;
    inherited MouseUp(Button, Shift, X, Y);
  end;
end;

procedure TOfficeButton.Paint;
var
  GlyphRect, BitmapRect: TRect;
  GlyphTop, GlyphLeft: integer;
begin
  LockWindowUpdate(Canvas.Handle);
  inherited Paint;

  GlyphTop := ((Height - FGlyph.Height) div 2) + ExtraSpace;
  GlyphLeft := 7 + ExtraSpace;
  GlyphRect := Rect(GlyphLeft, GlyphTop, FGlyph.Width+GlyphLeft, FGlyph.Height+GlyphTop);
  BitmapRect := Rect(0, 0, FGlyph.Width, FGlyph.Height);
  with Canvas do
  begin
    Brush.Style := bsClear;
    if Transparent then Brush.Color := Color else Brush.Color := FButtonColor;
    Font.Assign(FFont);

    if HasMouse then
    begin
      Pen.Color := BorderColor;
      Pen.Width := BorderWidth+1;
      RoundRect(BorderWidth,BorderWidth,Width-(BorderWidth-1),Height-(BorderWidth-1),FRoundSize,FRoundSize);

      Pen.Color := clWhite;
      Pen.Width := 2;
      MoveTo(FRoundSize div 2, BorderWidth+2);
      LineTo(Width - (FRoundSize div 2), BorderWidth+2);
    end
    else
    begin
      Pen.Color := clSilver;
      Pen.Width := BorderWidth;
      RoundRect(BorderWidth-1,BorderWidth-1,Width-(BorderWidth-1),Height-(BorderWidth-1),FRoundSize,FRoundSize);
    end;

    if HasMouse then
    begin
       if IsMouseDown then
          FGlyph.Handle := LoadBitmap(HInstance, 'DOWN')
       else
          FGlyph.Handle := LoadBitmap(HInstance, 'ON');
    end
    else
       FGlyph.Handle := LoadBitmap(HInstance, 'OFF');

    BrushCopy(GlyphRect, FGlyph, BitmapRect, clWhite);

    if not FEnabled then Font.Color := clGray;
    TextOut(((Width - TextWidth(FCaption)) div 2)+ GlyphLeft,((Height - TextHeight(FCaption)) div 2)+ExtraSpace,FCaption);
  end;
  LockWindowUpdate(0);
end;

constructor TOfficeButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle + [csOpaque];
  FFont := TFont.Create;
  {$IFNDEF WIN32}
    FFont.Name := 'MS Sans Serif';
    FFont.Size := 8;
    FFont.Style := [];
  {$ENDIF}
  FButtonColor := clSilver;
  FTransparent := true;
  Width := 100;
  Height := 25;
  FCaption := 'OfficeButton';
  FEnabled := true;
  FGlyph := TBitmap.Create;
  FGlyph.Handle := LoadBitmap(HInstance, 'OFF');
  FRoundSize := 20;
  HasMouse := false;
  IsMouseDown := false;
  ExtraSpace := 0;
  FBorderColor := clGray;
  FBorderWidth := 1;
end;

destructor TOfficeButton.Destroy;
begin
  FFont.Free;
  FGlyph.Free;
  inherited Destroy;
end;

{ Here starts the TOfficeLabel-code }

procedure TOfficeLabel.SetCaption(value: TCaption);
begin
  if value <> FCaption then
  begin
    FCaption := value;
    Repaint;
  end;
end;

procedure TOfficeLabel.SetEnabled(value: boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := value;
    HasMouse := false;
    IsMouseDown := false;
    IsClicked := false;
    Repaint;
  end;
end;

procedure TOfficeLabel.SetFont(value: TFont);
begin
  FFont.Assign(value);
  Repaint;
end;

procedure TOfficeLabel.SetLabelColor(value: TColor);
begin
  if Value <> FLabelColor then
  begin
    FLabelColor := Value;
    Repaint;
  end;
end;

procedure TOfficeLabel.SetLEDType(value: TLEDType);
begin
  if value <> FLEDType then
  begin
    FLEDType := value;
    case FLEDType of
         RoundLED:   FGlyph.Handle := LoadBitmap(HInstance, 'OFFRND');
         TriLEDDown: FGlyph.Handle := LoadBitmap(HInstance, 'OFF3DWN');
         TriLEDUp:   FGlyph.Handle := LoadBitmap(HInstance, 'OFF3UP');
    end;
    Repaint;
  end;
end;

procedure TOfficeLabel.SetTransparent(value: boolean);
begin
  if Value <> FTransparent then
  begin
    FTransparent := Value;
    Repaint;
  end;
end;

procedure TOfficeLabel.Click;
begin
  if Enabled then
  begin
    isClicked := true;
    inherited Click;
  end;
end;

procedure TOfficeLabel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Enabled then
  begin
    HasMouse := true;
    IsMouseDown := true;
    Repaint;
    inherited MouseDown(Button, Shift, X, Y);
  end;
end;

procedure TOfficeLabel.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Enabled then
  begin
    IsMouseDown := false;
    Repaint;
    inherited MouseUp(Button, Shift, X, Y);
  end;
end;

procedure TOfficeLabel.CMMouseEnter(var Message: TMessage);
begin
  if Enabled then
  begin
    HasMouse := true;
    Repaint;
    if Assigned(FMouseEnter) then FMouseEnter(self);
  end;
end;

procedure TOfficeLabel.CMMouseLeave(var Message: TMessage);
begin
  if Enabled then
  begin
    isClicked := true;
    HasMouse := false;
    Repaint;
    if Assigned(FMouseLeave) then FMouseLeave(self);
  end;
end;

procedure TOfficeLabel.WMMouseMove(var Message: TWMMouseMove);
var
  CurrentPoint: TPoint;
begin
  if Enabled then
  begin
    CurrentPoint.X := Message.XPos;
    CurrentPoint.Y := Message.YPos;
    if not HasMouse and PtInRect(GetClientRect, CurrentPoint) then
    begin
      HasMouse := true;
      Repaint;
    end;
  end;
  inherited;
end;

procedure TOfficeLabel.WMLButtonDblClick(var Message: TWMLButtonDown);
begin
  IsMouseDown := false;
  if Enabled then Inherited;
end;

procedure TOfficeLabel.WMMButtonDblClick(var Message: TWMMButtonDown);
begin
  IsMouseDown := false;
  if Enabled then Inherited;
end;

procedure TOfficeLabel.WMRButtonDblClick(var Message: TWMRButtonDown);
begin
  IsMouseDown := false;
  if Enabled then Inherited;
end;

procedure TOfficeLabel.Paint;
var
  GlyphRect, CaptionRect: TRect;
  CaptionLeft: word;
  PCaption: PChar;
begin
  LockWindowUpdate(Canvas.Handle);
  inherited Paint;
  GlyphRect := Rect(0, 0, FGlyph.Width, FGlyph.Height);

  with Canvas do
  begin
    if FTransparent then
    begin
       brush.style := bsClear;
       brush.color := Color;
    end
    else
    begin
      brush.style := bsSolid;
      brush.color := FLabelColor;
      pen.style := psClear;
      Rectangle(GetClientRect.Left, GetClientRect.Top, GetClientRect.Right, GetClientRect.Bottom);
    end;
    Font.Assign(FFont);

    if HasMouse then
    begin
       if IsMouseDown then
       begin
          case FLEDType of
               RoundLED:   FGlyph.Handle := LoadBitmap(HInstance, 'DOWNRND');
               TriLEDDown: FGlyph.Handle := LoadBitmap(HInstance, 'DOWN3DWN');
               TriLEDUp:   FGlyph.Handle := LoadBitmap(HInstance, 'DOWN3UP');
          end;
       end
       else
       begin
          case FLEDType of
               RoundLED:   FGlyph.Handle := LoadBitmap(HInstance, 'ONRND');
               TriLEDDown: FGlyph.Handle := LoadBitmap(HInstance, 'ON3DWN');
               TriLEDUp:   FGlyph.Handle := LoadBitmap(HInstance, 'ON3UP');
          end;
       end;
    end
    else
    begin
       case FLEDType of
            RoundLED:   FGlyph.Handle := LoadBitmap(HInstance, 'OFFRND');
            TriLEDDown: FGlyph.Handle := LoadBitmap(HInstance, 'OFF3DWN');
            TriLEDUp:   FGlyph.Handle := LoadBitmap(HInstance, 'OFF3UP');
       end;
    end;

    BrushCopy(GlyphRect, FGlyph, GlyphRect, clYellow);

    CaptionLeft := FGlyph.Width + 5;
    CaptionRect := Rect(CaptionLeft, 1, ClientWidth, ClientHeight-1);

    if not Enabled then Font.Color := clGray;

    {$IFDEF WIN32}
      DrawText(Handle, PChar(FCaption), Length(FCaption), CaptionRect, DT_WORDBREAK or DT_NOPREFIX);
    {$ELSE}
      if MaxAvail > SizeOf(FCaption) then
      begin
        GetMem(PCaption, SizeOf(FCaption));
        StrPCopy(PCaption, FCaption);
        DrawText(Handle, PCaption, StrLen(PCaption), CaptionRect, DT_WORDBREAK or DT_NOPREFIX);
        FreeMem(PCaption, SizeOf(FCaption));
      end;
    {$ENDIF}

    if (HasMouse) and (IsMouseDown) then
    begin
      Brush.Style := bsClear;
      Pen.Color := clBlack;
      Pen.Width := 1;
      Pen.Style := psDot;
      Pen.Mode := pmMask;
      Rectangle(CaptionRect.Left-3, CaptionRect.Top-1, CaptionRect.Right-1, CaptionRect.Bottom);
    end;

    if not (HasMouse) and (isClicked) then
    begin
      isClicked := false;
      Brush.Style := bsClear;
      if FTransparent then Pen.Color := Color else Pen.Color := LabelColor;
      Pen.Width := 1;
      Pen.Style := psDot;
      Pen.Mode := pmCopy;
      Rectangle(CaptionRect.Left-3, CaptionRect.Top-1, CaptionRect.Right-1, CaptionRect.Bottom);
    end;

  end;
  LockWindowUpdate(0);
end;

constructor TOfficeLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle + [csOpaque];
  width := 100;
  height := 17;
  FFont := TFont.Create;
  {$IFNDEF WIND32}
    FFont.Name := 'MS Sans Serif';
    FFont.Size := 8;
    FFont.Style := [];
  {$ENDIF}
  FCaption := 'OfficeLabel';
  FEnabled := true;
  FLabelColor := clSilver;
  FTransparent := True;
  LEDType := RoundLED;
  FGlyph := TBitmap.Create;
  FGlyph.Handle := LoadBitmap(HInstance, 'OFFRND');
end;

destructor TOfficeLabel.Destroy;
begin
  FFont.Free;
  FGlyph.Free;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Win95', [TOfficeButton, TOfficeLabel]);
end;

end.
